import { SEMAPHORE } from "@thi.ng/api/api";
import { diffArray } from "@thi.ng/diff/array";
import { diffObject } from "@thi.ng/diff/object";
import {
  equiv as _equiv,
  equivArrayLike,
  equivMap,
  equivObject,
  equivSet
} from "@thi.ng/equiv";
const isArray = Array.isArray;
const max = Math.max;
const OBJP = Object.getPrototypeOf({});
const FN = "function";
const STR = "string";
const INDEX = (() => {
  const res = new Array(2048);
  for (let i = 2, n = res.length; i < n; i++) {
    res[i] = i - 2;
  }
  return res;
})();
const __buildIndex = (n) => {
  if (n <= INDEX.length) {
    return INDEX.slice(0, n);
  }
  const res = new Array(n);
  while (n-- > 2) {
    res[n] = n - 2;
  }
  return res;
};
const diffTree = (opts, impl, parent, prev, curr, child = 0) => {
  const attribs = curr[1];
  if (attribs.__skip) {
    return;
  }
  if (attribs.__diff === false) {
    releaseTree(prev);
    impl.replaceChild(opts, parent, child, curr);
    return;
  }
  const pattribs = prev[1];
  if (pattribs?.__skip) {
    impl.replaceChild(opts, parent, child, curr, false);
    return;
  }
  let _impl = attribs.__impl;
  if (_impl && _impl !== impl) {
    return _impl.diffTree(opts, _impl, parent, prev, curr, child);
  }
  const delta = diffArray(prev, curr, "only-distance-linear", equiv);
  if (delta.distance === 0) {
    return;
  }
  const edits = delta.linear;
  const el = impl.getChild(parent, child);
  let i;
  let ii;
  let status;
  let val;
  if (edits[0] !== 0 || prev[1].key !== attribs.key) {
    releaseTree(prev);
    impl.replaceChild(opts, parent, child, curr);
    return;
  }
  if ((val = prev.__release) && val !== curr.__release) {
    releaseTree(prev);
  }
  if (edits[3] !== 0) {
    diffAttributes(impl, el, prev[1], curr[1]);
    if (delta.distance === 2) {
      return;
    }
  }
  const numEdits = edits.length;
  const prevLength = prev.length - 1;
  const equivKeys = __extractEquivElements(edits);
  const offsets = __buildIndex(prevLength + 1);
  for (i = 2, ii = 6; ii < numEdits; i++, ii += 3) {
    status = edits[ii];
    if (!status) continue;
    if (status === -1) {
      __diffDeleted(
        opts,
        impl,
        el,
        prev,
        curr,
        edits,
        ii,
        equivKeys,
        offsets,
        prevLength
      );
    } else {
      __diffAdded(
        opts,
        impl,
        el,
        edits,
        ii,
        equivKeys,
        offsets,
        prevLength
      );
    }
  }
  if ((val = curr.__init) && val != prev.__init) {
    val.apply(curr, [el, ...curr.__args]);
  }
};
const __diffDeleted = (opts, impl, el, prev, curr, edits, ii, equivKeys, offsets, prevLength) => {
  const val = edits[ii + 2];
  if (isArray(val)) {
    let k = val[1].key;
    if (k !== void 0 && equivKeys[k][2] !== void 0) {
      const eq = equivKeys[k];
      k = eq[0];
      diffTree(opts, impl, el, prev[k], curr[eq[2]], offsets[k]);
    } else {
      const idx = edits[ii + 1];
      releaseTree(val);
      impl.removeChild(el, offsets[idx]);
      __incOffsets(offsets, prevLength, idx);
    }
  } else if (typeof val === STR) {
    impl.setContent(el, "");
  }
};
const __diffAdded = (opts, impl, el, edits, ii, equivKeys, offsets, prevLength) => {
  const val = edits[ii + 2];
  if (typeof val === STR) {
    impl.setContent(el, val);
  } else if (isArray(val)) {
    const k = val[1].key;
    if (k === void 0 || equivKeys[k][0] === void 0) {
      const idx = edits[ii + 1];
      impl.createTree(opts, el, val, offsets[idx]);
      __decOffsets(offsets, prevLength, idx);
    }
  }
};
const __incOffsets = (offsets, j, idx) => {
  for (; j > idx; j--) {
    offsets[j] = max(offsets[j] - 1, 0);
  }
};
const __decOffsets = (offsets, j, idx) => {
  for (; j >= idx; j--) {
    offsets[j]++;
  }
};
const diffAttributes = (impl, el, prev, curr) => {
  const delta = diffObject(prev, curr, "full", _equiv);
  impl.removeAttribs(el, delta.dels, prev);
  let val = SEMAPHORE;
  let i, e, edits;
  for (edits = delta.edits, i = edits.length; (i -= 2) >= 0; ) {
    e = edits[i];
    e[0] === "o" && e[1] === "n" && impl.removeAttribs(el, [e], prev);
    e !== "value" ? impl.setAttrib(el, e, edits[i + 1], curr) : val = edits[i + 1];
  }
  for (edits = delta.adds, i = edits.length; i-- > 0; ) {
    e = edits[i];
    e !== "value" ? impl.setAttrib(el, e, curr[e], curr) : val = curr[e];
  }
  val !== SEMAPHORE && impl.setAttrib(el, "value", val, curr);
};
const releaseTree = (tree) => {
  if (isArray(tree)) {
    let x;
    if ((x = tree[1]) && x.__release === false) {
      return;
    }
    if (tree.__release) {
      tree.__release.apply(tree.__this, tree.__args);
      delete tree.__release;
    }
    for (x = tree.length; x-- > 2; ) {
      releaseTree(tree[x]);
    }
  }
};
const __extractEquivElements = (edits) => {
  let k;
  let val;
  let ek;
  const equiv2 = {};
  for (let i = edits.length; (i -= 3) >= 0; ) {
    val = edits[i + 2];
    if (isArray(val) && (k = val[1].key) !== void 0) {
      ek = equiv2[k];
      !ek && (equiv2[k] = ek = [, ,]);
      ek[edits[i] + 1] = edits[i + 1];
    }
  }
  return equiv2;
};
const equiv = (a, b) => {
  let proto;
  if (a === b) {
    return true;
  }
  if (a != null) {
    if (typeof a.equiv === FN) {
      return a.equiv(b);
    }
  } else {
    return a == b;
  }
  if (b != null) {
    if (typeof b.equiv === FN) {
      return b.equiv(a);
    }
  } else {
    return a == b;
  }
  if (typeof a === STR || typeof b === STR) {
    return false;
  }
  if ((proto = Object.getPrototypeOf(a), proto == null || proto === OBJP) && (proto = Object.getPrototypeOf(b), proto == null || proto === OBJP)) {
    return !(a.__diff === false || b.__diff === false) && equivObject(a, b, equiv);
  }
  if (typeof a !== FN && a.length !== void 0 && typeof b !== FN && b.length !== void 0) {
    return equivArrayLike(a, b, equiv);
  }
  if (a instanceof Set && b instanceof Set) {
    return equivSet(a, b, equiv);
  }
  if (a instanceof Map && b instanceof Map) {
    return equivMap(a, b, equiv);
  }
  if (a instanceof Date && b instanceof Date) {
    return a.getTime() === b.getTime();
  }
  if (a instanceof RegExp && b instanceof RegExp) {
    return a.toString() === b.toString();
  }
  return a !== a && b !== b;
};
export {
  diffAttributes,
  diffTree,
  equiv,
  releaseTree
};
