import { implementsFunction } from "@thi.ng/checks/implements-function";
import { isArray } from "@thi.ng/checks/is-array";
import { isNotStringAndIterable } from "@thi.ng/checks/is-not-string-iterable";
import { isString } from "@thi.ng/checks/is-string";
import { ATTRIB_JOIN_DELIMS, SVG_TAGS } from "@thi.ng/hiccup/api";
import { css } from "@thi.ng/hiccup/css";
import { formatPrefixes } from "@thi.ng/hiccup/prefix";
import { XML_SVG } from "@thi.ng/prefixes/xml";
const __maybeInitElement = (el, tree) => tree.__init?.apply(tree.__this, [el, ...tree.__args]);
const createTree = (opts, impl, parent, tree, insert, init = true) => {
  if (isArray(tree)) {
    const tag = tree[0];
    if (typeof tag === "function") {
      return createTree(
        opts,
        impl,
        parent,
        tag.apply(null, [opts.ctx, ...tree.slice(1)]),
        insert
      );
    }
    const attribs = tree[1];
    if (attribs.__impl) {
      return attribs.__impl.createTree(
        opts,
        parent,
        tree,
        insert,
        init
      );
    }
    const el = impl.createElement(parent, tag, attribs, insert);
    if (tree.length > 2) {
      const n = tree.length;
      for (let i = 2; i < n; i++) {
        createTree(opts, impl, el, tree[i], void 0, init);
      }
    }
    init && __maybeInitElement(el, tree);
    return el;
  }
  if (isNotStringAndIterable(tree)) {
    const res = [];
    for (let t of tree) {
      res.push(createTree(opts, impl, parent, t, insert, init));
    }
    return res;
  }
  if (tree == null) {
    return parent;
  }
  return impl.createTextElement(parent, tree);
};
const hydrateTree = (opts, impl, parent, tree, index = 0) => {
  if (isArray(tree)) {
    const el = impl.getChild(parent, index);
    if (typeof tree[0] === "function") {
      hydrateTree(
        opts,
        impl,
        parent,
        tree[0].apply(null, [opts.ctx, ...tree.slice(1)]),
        index
      );
    }
    const attribs = tree[1];
    if (attribs.__impl) {
      return attribs.__impl.hydrateTree(
        opts,
        parent,
        tree,
        index
      );
    }
    __maybeInitElement(el, tree);
    for (let a in attribs) {
      a[0] === "o" && a[1] === "n" && impl.setAttrib(el, a, attribs[a]);
    }
    for (let n = tree.length, i = 2; i < n; i++) {
      hydrateTree(opts, impl, el, tree[i], i - 2);
    }
  } else if (isNotStringAndIterable(tree)) {
    for (let t of tree) {
      hydrateTree(opts, impl, parent, t, index);
      index++;
    }
  }
};
const createElement = (parent, tag, attribs, insert) => {
  const el = SVG_TAGS[tag] ? document.createElementNS(XML_SVG, tag) : document.createElement(tag);
  attribs && setAttribs(el, attribs);
  return addChild(parent, el, insert);
};
const createTextElement = (parent, content, insert) => addChild(parent, document.createTextNode(content), insert);
const addChild = (parent, child, insert) => parent ? insert === void 0 ? parent.appendChild(child) : parent.insertBefore(child, parent.children[insert]) : child;
const getChild = (parent, child) => parent.children[child];
const replaceChild = (opts, impl, parent, child, tree, init = true) => (impl.removeChild(parent, child), impl.createTree(opts, parent, tree, child, init));
const cloneWithNewAttribs = (el, attribs) => {
  const res = el.cloneNode(true);
  setAttribs(res, attribs);
  el.parentNode.replaceChild(res, el);
  return res;
};
const setContent = (el, body) => el.textContent = body;
const setAttribs = (el, attribs) => {
  for (let k in attribs) {
    setAttrib(el, k, attribs[k], attribs);
  }
  return el;
};
const setAttrib = (el, id, val, attribs) => {
  implementsFunction(val, "deref") && (val = val.deref());
  if (id.startsWith("__")) return;
  const isListener = id[0] === "o" && id[1] === "n";
  if (isListener) {
    if (isString(val)) {
      el.setAttribute(id, val);
    } else {
      id = id.substring(2);
      isArray(val) ? el.addEventListener(id, val[0], val[1]) : el.addEventListener(id, val);
    }
    return el;
  }
  if (typeof val === "function") val = val(attribs);
  if (isArray(val)) val = val.join(ATTRIB_JOIN_DELIMS[id] || " ");
  switch (id) {
    case "style":
      setStyle(el, val);
      break;
    case "value":
      updateValueAttrib(el, val);
      break;
    case "prefix":
      el.setAttribute(id, isString(val) ? val : formatPrefixes(val));
      break;
    case "accesskey":
    case "accessKey":
      el.accessKey = val;
      break;
    case "contenteditable":
    case "contentEditable":
      el.contentEditable = val;
      break;
    case "tabindex":
    case "tabIndex":
      el.tabIndex = val;
      break;
    case "align":
    case "autocapitalize":
    case "checked":
    case "dir":
    case "draggable":
    case "hidden":
    case "id":
    case "indeterminate":
    case "lang":
    case "namespaceURI":
    case "scrollLeft":
    case "scrollTop":
    case "selectionEnd":
    case "selectionStart":
    case "slot":
    case "spellcheck":
    case "title":
      el[id] = val;
      break;
    default:
      val === false || val == null ? el.removeAttribute(id) : el.setAttribute(id, val === true ? id : val);
  }
  return el;
};
const updateValueAttrib = (el, value) => {
  let ev;
  switch (el.type) {
    case "text":
    case "textarea":
    case "password":
    case "search":
    case "number":
    case "email":
    case "url":
    case "tel":
    case "date":
    case "datetime-local":
    case "time":
    case "week":
    case "month":
      if ((ev = el.value) !== void 0 && typeof value === "string") {
        const off = value.length - (ev.length - (el.selectionStart || 0));
        el.value = value;
        el.selectionStart = el.selectionEnd = off;
        break;
      }
    default:
      el.value = value;
  }
};
const removeAttribs = (el, attribs, prev) => {
  for (let i = attribs.length; i-- > 0; ) {
    const a = attribs[i];
    if (a[0] === "o" && a[1] === "n") {
      removeListener(el, a.substring(2), prev[a]);
    } else {
      el.hasAttribute(a) ? el.removeAttribute(a) : el[a] = null;
    }
  }
};
const setStyle = (el, styles) => (el.setAttribute("style", css(styles)), el);
const setListener = (el, id, listener) => isString(listener) ? el.setAttribute("on" + id, listener) : isArray(listener) ? el.addEventListener(id, ...listener) : el.addEventListener(id, listener);
const removeListener = (el, id, listener) => isArray(listener) ? el.removeEventListener(id, ...listener) : el.removeEventListener(id, listener);
const clearDOM = (el) => el.innerHTML = "";
const removeChild = (parent, childIdx) => {
  const n = parent.children[childIdx];
  n !== void 0 && parent.removeChild(n);
};
export {
  addChild,
  clearDOM,
  cloneWithNewAttribs,
  createElement,
  createTextElement,
  createTree,
  getChild,
  hydrateTree,
  removeAttribs,
  removeChild,
  removeListener,
  replaceChild,
  setAttrib,
  setAttribs,
  setContent,
  setListener,
  setStyle,
  updateValueAttrib
};
