import { isArray as isa } from "@thi.ng/checks/is-array";
import { isNotStringAndIterable as isi } from "@thi.ng/checks/is-not-string-iterable";
import { isPlainObject as iso } from "@thi.ng/checks/is-plain-object";
import { illegalArgs } from "@thi.ng/errors/illegal-arguments";
import { NO_SPANS, RE_TAG } from "@thi.ng/hiccup/api";
import { mergeEmmetAttribs } from "@thi.ng/hiccup/attribs";
const isArray = isa;
const isNotStringAndIterable = isi;
const isPlainObject = iso;
const normalizeElement = (spec, keys) => {
  let tag = spec[0];
  let hasAttribs = isPlainObject(spec[1]);
  let match;
  let name;
  let attribs;
  if (typeof tag !== "string" || !(match = RE_TAG.exec(tag))) {
    illegalArgs(`${tag} is not a valid tag name`);
  }
  name = match[1];
  if (tag === name && hasAttribs && (!keys || spec[1].key)) {
    return spec;
  }
  attribs = mergeEmmetAttribs(
    hasAttribs ? { ...spec[1] } : {},
    match[2],
    match[3]
  );
  return attribs.__skip && spec.length < 3 ? [name, attribs] : [name, attribs, ...spec.slice(hasAttribs ? 2 : 1)];
};
const normalizeTree = (opts, tree) => __normalizeTree(
  tree,
  opts,
  opts.ctx,
  [0],
  opts.keys !== false,
  opts.span !== false
);
const __normalizeTree = (tree, opts, ctx, path, keys, span) => tree == null ? void 0 : isArray(tree) ? __normalizeArray(tree, opts, ctx, path, keys, span) : typeof tree === "function" ? __normalizeTree(tree(ctx), opts, ctx, path, keys, span) : typeof tree.toHiccup === "function" ? __normalizeTree(tree.toHiccup(opts.ctx), opts, ctx, path, keys, span) : typeof tree.deref === "function" ? __normalizeTree(tree.deref(), opts, ctx, path, keys, span) : span ? ["span", keys ? { key: path.join("-") } : {}, tree.toString()] : tree.toString();
const __normalizeArray = (tree, opts, ctx, path, keys, span) => {
  if (tree.length === 0) return;
  let norm, nattribs = tree[1], impl;
  if (nattribs && (impl = nattribs.__impl) && (impl = impl.normalizeTree)) {
    return impl(opts, tree);
  }
  const tag = tree[0];
  if (typeof tag === "function") {
    return __normalizeTree(
      tag.apply(null, [ctx, ...tree.slice(1)]),
      opts,
      ctx,
      path,
      keys,
      span
    );
  }
  if (typeof tag.render === "function") {
    const args = [ctx, ...tree.slice(1)];
    norm = __normalizeTree(
      tag.render.apply(tag, args),
      opts,
      ctx,
      path,
      keys,
      span
    );
    if (isArray(norm)) {
      norm.__this = tag;
      norm.__init = tag.init;
      norm.__release = tag.release;
      norm.__args = args;
    }
    return norm;
  }
  norm = normalizeElement(tree, keys);
  nattribs = norm[1];
  if (nattribs.__normalize === false) {
    return norm;
  }
  if (keys && nattribs.key === void 0) {
    nattribs.key = path.join("-");
  }
  return norm.length > 2 ? __normalizeChildren(norm, nattribs, opts, ctx, path, keys, span) : norm;
};
const __normalizeChildren = (norm, nattribs, opts, ctx, path, keys, span) => {
  const tag = norm[0];
  const res = [tag, nattribs];
  span = span && !NO_SPANS[tag];
  for (let i = 2, j = 2, k = 0, n = norm.length; i < n; i++) {
    let el = norm[i];
    if (el != null) {
      const isarray = isArray(el);
      if (isarray && isArray(el[0]) || !isarray && isNotStringAndIterable(el)) {
        for (let c of el) {
          c = __normalizeTree(
            c,
            opts,
            ctx,
            path.concat(k),
            keys,
            span
          );
          if (c !== void 0) {
            res[j++] = c;
          }
          k++;
        }
      } else {
        el = __normalizeTree(el, opts, ctx, path.concat(k), keys, span);
        if (el !== void 0) {
          res[j++] = el;
        }
        k++;
      }
    }
  }
  return res;
};
export {
  normalizeElement,
  normalizeTree
};
